/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import java.io.Serializable;


/**
 * <p>
 * Defines a structure which holds the information for the url and page title of one specific bread crumb node.
 * </p>
 *
 * <p>
 * This structure will be used in <code>NodeDiscovery</code> class to get the path from the current node to the root
 * node. Each node in that path will be respresented by this structure.
 * </p>
 *
 * <p>
 * <b>Changes in Version 2.0 :</b>
 *
 * <ol>
 * <li>
 * Eliminated the parentURL from this class (variable, constructor arg and getter/setter).
 * </li>
 * <li>
 * Eliminated all setter methods (making this class immutable).
 * </li>
 * <li>
 * Updated constructor to throw <code>IllegalArgumentException</code> if url string is <code>null</code>.
 * </li>
 * <li>
 * Eliminated synchronization.
 * </li>
 * </ol>
 * </p>
 *
 * <p>
 * <b>Thread Safety: </b> This class is immutable and therefore thread safe.
 * </p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 *
 * @since 1.0
 */
public class BreadCrumbNode implements Serializable {
    /**
     * <p>
     * Represents the title associated with this node.
     * </p>
     *
     * <p>
     * This variable is set by the constructor and is immutable. This value can be <code>null</code> if the title
     * should default to the title found in the <code>PathDiscovery</code> implementation. This value should not be
     * <code>null</code> if it's a node that is part of a path returned by the <code>PathDiscovery</code>
     * implementation. This value should never be an empty string.
     * </p>
     */
    private final String title;

    /**
     * <p>
     * Represents the url (including query string) associated with this node.
     * </p>
     *
     * <p>
     * This variable is set by the constructor, is immutable and will never be <code>null</code>.
     * </p>
     */
    private final String url;

    /**
     * <p>
     * Creates a new instance of <code>BreadCrumbNode</code> class with given node url and title.
     * </p>
     *
     * <p>
     * <b>Changes in Version 2.0 :</b> Updated constructor to throw <code>IllegalArgumentException</code> if url string
     * is <code>null</code> and eliminated the parentURL from this constructor.
     * </p>
     *
     * @param url the url of page represented by this node.
     * @param title the title of page represented by this node. If the title is an empty string, it will be regarded as
     *        <code>null</code>.
     *
     * @throws IllegalArgumentException if <code>url</code> is <code>null</code> or an empty (trim'd) string.
     *
     * @since 1.0
     */
    public BreadCrumbNode(String url, String title) {
        Helper.validateString(url, "url");
        this.url = url;
        this.title = Helper.convertEmptyString(title);
    }

    /**
     * <p>
     * Gets the title of page represented by this node.
     * </p>
     *
     * <p>
     * <b>Changes in Version 2.0 :</b> Eliminated synchronization.
     * </p>
     *
     * @return A possibly <code>null</code>, non-empty (trim'd) string representing the title of page represented by
     *         this node.
     *
     * @since 1.0
     */
    public String getTitle() {
        return this.title;
    }

    /**
     * <p>
     * Gets the url (including query string) of page represented by this node.
     * </p>
     *
     * <p>
     * <b>Changes in Version 2.0 :</b> Eliminated synchronization.
     * </p>
     *
     * @return A non-null, non-empty (trim'd) string representing the url (including query string) of page represented
     *         by this node.
     *
     * @since 1.0
     */
    public String getUrl() {
        return this.url;
    }
}
