/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.nodeformatter;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbTrailTag;
import com.topcoder.web.ui.tag.breadcrumbtrail.Helper;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.TagSupport;


/**
 * <p>
 * This tag will format the node and style using a template.&nbsp; A template is a separate file that will be included
 * in the out stream.&nbsp; This class will take the current node/style and define them as request attributes when the
 * template file is included (those attributes can then be used by the template file to format the correct
 * representation of them).
 * </p>
 *
 * <p>
 * <strong>Thread Safety:</strong> This class is thread safe in the context of the JSP rendering process.
 * </p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 */
public class TemplateNodeFormatterTag extends TagSupport {
    /**
     * <p>
     * Represents the template file that will be used when formatting this tag. This variable is set by the
     * constructor, is mutable and can be null or an empty (trim'd) string to specify template is to be used (and thus
     * nothing produced by this tag).&nbsp; Please note that it is expected (but not required) that the value will be
     * a non-null, non-empty value.&nbsp.
     * </p>
     */
    private String template = null;

    /**
     * <p>
     * Creates a new instance of <code>TemplateNodeFormatterTag</code> class.
     * </p>
     */
    public TemplateNodeFormatterTag() {
        super();
    }

    /**
     * <p>
     * Creates a new instance of <code>TemplateNodeFormatterTag</code> class with given template.
     * </p>
     *
     * @param template A possibly null, possibly empty (trim'd) string representing the template to use.
     */
    public TemplateNodeFormatterTag(String template) {
        this.template = template;
    }

    /**
     * Gets the template file that will be used when formatting this tag.
     *
     * @return A possibly null, possibly empty (trim'd) string representing the template file that will be used when
     *         formatting this tag.
     */
    public String getTemplate() {
        return template;
    }

    /**
     * Sets the template file that will be used when formatting this tag.
     *
     * @param template A possibly null, possibly empty (trim'd) string representing the template file that will be used
     *        when formatting this tag.
     */
    public void setTemplate(String template) {
        this.template = template;
    }

    /**
     * <p>
     * This method will take the current node and format, make them into request attributes and include the template
     * file (which is assumed to use the request attributes).
     * </p>
     *
     * @return Always SKIP_BODY.
     *
     * @throws BreadCrumbException if some exception occurs during processing.
     */
    public int doStartTag() throws BreadCrumbException {
        if ((template == null) || (template.trim().length() == 0)) {
            return SKIP_BODY;
        }

        // get the node to format
        Object node = pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, PageContext.PAGE_SCOPE);

        if (!(node instanceof BreadCrumbNode)) {
            throw new BreadCrumbException("The node to format does not exist in the pageContext.");
        }

        // get the style to format
        Object style = pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS,
                PageContext.PAGE_SCOPE);

        if ((style != null) && !(style instanceof String)) {
            throw new BreadCrumbException("The style to format is not a string.");
        }

        // if the style is an empty string, move null to it instead.
        style = Helper.convertEmptyString((String) style);

        // get the request from the page context
        ServletRequest request = pageContext.getRequest();

        // set the node attribute and style attribute
        request.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, node);
        request.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, style);

        try {
            // include the template
            pageContext.include(template);
        } catch (ServletException e) {
            throw new BreadCrumbException("The page that was forwarded to throws a ServletException.", e);
        } catch (IOException e) {
            throw new BreadCrumbException("An I/O error occurred while forwarding.", e);
        }

        return SKIP_BODY;
    }
}
