/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.Helper;

import java.io.Serializable;


/**
 * <p>
 * This class is a typical pair class that holds a paired association of a node matcher to the matching node. The
 * nodeMatcher describes the node that matched the above node and describes the default node information. The node
 * describes the actual node that matched the nodeMatcher and contains the contextual specific information.
 * </p>
 *
 * <p>
 * <strong>Thread Safety: </strong> This class is thread safe by being immutable.
 * </p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 */
class NodePair implements Serializable {
    /**
     * <p>
     * Represents the actual node that holds the contextual information (query string, etc) that matches the node
     * matcher. This variable is set  by the constructor, is immutable and can never be null.
     * </p>
     */
    private final BreadCrumbNode node;

    /**
     * <p>
     * Represents the node matcher that holds the matching information and the default title. This variable is set  by
     * the constructor, is immutable and can be null (if there is no associated nodeMatcher).
     * </p>
     */
    private final NodeMatcher nodeMatcher;

    /**
     * <p>
     * Creates a new instance of <code>NodePair</code> class.
     * </p>
     *
     * @param node The non-null bread crumb node that matches the path node.
     * @param nodeMatcher The possibly null node matcher.
     *
     * @throws IllegalArgumentException if node is null.
     */
    NodePair(BreadCrumbNode node, NodeMatcher nodeMatcher) {
        Helper.validateNotNull(node, "node");
        this.node = node;
        this.nodeMatcher = nodeMatcher;
    }

    /**
     * <p>
     * Gets the node. This method will simply return the node if the node's title is non-null. If the node's title is
     * null, a new node will be created with the title from the node matcher (if there is one)
     * </p>
     *
     * @return A non-null node.
     */
    public BreadCrumbNode getNode() {
        if (node.getTitle() != null) {
            return node;
        }

        if (nodeMatcher != null) {
            return new BreadCrumbNode(node.getUrl(), nodeMatcher.getTitle());
        }

        // if title of node is null and nodeMatcher is null either, return the original node
        return node;
    }

    /**
     * <p>
     * Gets the node matcher.
     * </p>
     *
     * @return A possibly null node matcher.
     */
    public NodeMatcher getNodeMatcher() {
        return nodeMatcher;
    }
}
