/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.Helper;
import com.topcoder.web.ui.tag.breadcrumbtrail.PathDiscovery;
import com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.nodematchers.RegexNodeMatcher;

import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.servlet.jsp.PageContext;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;


/**
 * <p>
 * This implementation of the PathDiscovery <strong>and </strong>SiteDiscovery will attempt to create a static site map
 * from either a filename or string source (both of which is an XML document).&nbsp; This class will parse the
 * document to create a static map of Regex matchers (where each node can provide a regex matching).&nbsp; If used as
 * a PathDiscovery, this provides a static view of the site (ie no dynamic capabilities).
 * </p>
 *
 * <p>
 * <strong>Thread Safety:</strong> Implementations of this interface should be thread safe.
 * </p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 */
public class XmlPathDiscovery implements SiteDiscovery, PathDiscovery {
    /**
     * <p>
     * Represents the session scope attribute name that will contain the site map. This constant will be used by this
     * class to retrieve the site map. The site map will be a <code>Map</code> with <code>NodeMatcher</code> as the
     * key and a list of <code>NodeMatcher</code> as the value and can be empty. This map represents the
     * <code>NodeMatcher</code> parent to it's children NodeMatcher(s) if any. The list can be empty if there are no
     * children of the parent (ie a leaf node).
     * </p>
     */
    public static final String CURRENT_SITEMAP = XmlPathDiscovery.class.getName() + ".siteMap";

    /**
     * <p>
     * Represents the session scope attribute name that will contain the reverse site map. This constant will be used
     * by this class to retrieve the reverse site map. The reverse site map will be a <code>Map</code> with
     * <code>NodeMatcher</code> as the key and a list of <code>NodeMatcher</code> as the value and can not be empty.
     * This map represents the <code>NodeMatcher</code> child to it's parents NodeMatcher(s) if any.
     * </p>
     */
    public static final String CURRENT_REVERSE_SITEMAP = XmlPathDiscovery.class.getName() + ".currentReverseSiteMap";

    /**
     * <p>
     * Represents the source of the site data.
     * </p>
     */
    private final String source;

    /**
     * <p>
     * Represents whether the source variable represents a file (true) or a raw xml string (false).
     * </p>
     */
    private final boolean isFile;

    /**
     * <p>
     * Creates a new instance of <code>XmlPathDiscovery</code> with given file name.
     * </p>
     *
     * @param filename A non-null, non-empty (trim'd) string representing the file to use
     *
     * @throws IllegalArgumentException if the filename is a null or empty (trim'd) string
     */
    public XmlPathDiscovery(String filename) {
        this(filename, true);
    }

    /**
     * <p>
     * Creates a new instance of <code>XmlPathDiscovery</code> with the given source and whether that source is a
     * direct XML string or a filename.
     * </p>
     *
     * @param source A non-null, non-empty (trim'd) string representing the source
     * @param isFile Whether the source is a file (true) or a string (false)
     *
     * @throws IllegalArgumentException if the source is null or an emtpy (trim'd) string
     */
    public XmlPathDiscovery(String source, boolean isFile) {
        Helper.validateString(source, "source");
        this.source = source;
        this.isFile = isFile;
    }

    /**
     * <p>
     * Returns a map of NodeMatcher keys with a List of NodeMatcher elements as the value.&nbsp; The key NodeMatcher
     * represents the parent and the value (the list) represents the child values.&nbsp; The value can be null or an
     * empty list to represent no child elements (ie it'a s leaf node).
     * </p>
     *
     * @return A non-null map of parent NodeMatcher to a list of NodeMatcher children
     *
     * @throws BreadCrumbException if parsing of data source failed.
     */
    public Map getSiteMap() throws BreadCrumbException {
        // create a DOM tree and add the specific node
        try {
            // create a new builder factory.
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

            // set validate xml and parse xml at the same time
            factory.setValidating(true);

            // get a document builder
            DocumentBuilder builder = factory.newDocumentBuilder();

            // set error handler. If the xml is well-formated but invalid, the handle class will throw exceptions.
            builder.setErrorHandler(new ErrorChecker());

            Reader reader = null;

            if (isFile) {
                // isFile is true, the source string represents the name of the file to load the xml string.
                reader = new FileReader(source);
            } else {
                // isFile is false, the source string represents the raw xml string.
                reader = new StringReader(source);
            }

            // parse the string into the xml document
            Document document = builder.parse(new InputSource(reader));

            // parse all the xml nodes into a map
            Element root = document.getDocumentElement();
            Map trailMap = new HashMap();
            parseChildren(null, root, trailMap);

            return trailMap;
        } catch (IOException e) {
            throw new BreadCrumbException("Error occurs when parsing file.", e);
        } catch (SAXException e) {
            throw new BreadCrumbException("Error occurs when parsing file or xml string.", e);
        } catch (ParserConfigurationException e) {
            throw new BreadCrumbException("Error occurs during creating Document for file or xml string.", e);
        } catch (DOMException e) {
            throw new BreadCrumbException("Error occurs during creating Document for file or xml string.", e);
        }
    }

    /**
     * Parses part of XML document which are children of node argument. All children are added to the trailMap as
     * <code>NodeMatcher</code> instances. Child nodes with a nodeName of bc_node will be added to the trailMap as
     * NodeMatcher parseChildren will be recursively called for all child nodes.
     *
     * @param currentMatcher the current node matcher.
     * @param node not for which children we look for bc_node nodes.
     * @param trailMap Map to which to add found nodes.
     *
     * @throws BreadCrumbException if the siteMap contains two parent nodes with the same name/title/url.
     */
    private void parseChildren(NodeMatcher currentMatcher, Node node, Map trailMap) throws BreadCrumbException {
        NodeList nodeList = node.getChildNodes();

        List breadCrumbNodes = new ArrayList();

        for (int i = 0; i < nodeList.getLength(); i++) {
            Node child = nodeList.item(i);

            if ((child.getNodeType() == Node.ELEMENT_NODE) && child.getNodeName().equals("bc_node")) {
                String url = ((Element) child).getAttribute("url");
                String title = ((Element) child).getAttribute("title");
                String pattern = ((Element) child).getAttribute("pattern");
                NodeMatcher matcher = new RegexNodeMatcher(title, url, pattern);
                breadCrumbNodes.add(matcher);
                parseChildren(matcher, child, trailMap);
            }
        }

        if (currentMatcher != null) {
            // whichever one contains the children is considered the 'unique' parent node, other wise it
            // will be regarded as invalid
            List oldBreadCrumbNodes = (List) trailMap.get(currentMatcher);

            if ((oldBreadCrumbNodes == null) || (oldBreadCrumbNodes.size() == 0)) {
                trailMap.put(currentMatcher, breadCrumbNodes);
            } else if (breadCrumbNodes.size() != 0) {
                throw new BreadCrumbException("The siteMap contains two parent nodes with the same name/title/url.");
            }
        }
    }

    /**
     * <p>
     * This method attempts to determine the page from the current node to the root and return that path based on the
     * current site map.
     * </p>
     *
     * @param currentNode A non-null current node
     * @param pageContext A non-null page context
     *
     * @return A null if the path can't be found or a non-empty array representing the path from root node (0th index)
     *         to current node (size-1th index)
     *
     * @throws IllegalArgumentException if any arg is null
     * @throws BreadCrumbException if an exception occurred while processing
     */
    public BreadCrumbNode[] getPath(BreadCrumbNode currentNode, PageContext pageContext) throws BreadCrumbException {
        Helper.validateNotNull(currentNode, "currentNode");
        Helper.validateNotNull(pageContext, "pageContext");

        // try to get the site map from the session attribute CURRENT_SITEMAP
        Object object = pageContext.getAttribute(CURRENT_SITEMAP, PageContext.SESSION_SCOPE);
        Map siteMap = null;

        if (!(object instanceof Map)) {
            // if not found or the attribute value is not of Map type, parse the siteMap from the source
            siteMap = this.getSiteMap();

            // save the siteMap to the session attribute CURRENT_SITEMAP
            pageContext.setAttribute(CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);
        } else {
            siteMap = (Map) object;
        }

        // get the shortest path to a root node
        NodeMatcher[] shortestPath = this.getShortestPath(pageContext, siteMap, currentNode);

        if (shortestPath == null) {
            // if the shortest path was not found, simply return null
            return null;
        }

        // return the shortest path
        BreadCrumbNode[] breadCrumbNodes = new BreadCrumbNode[shortestPath.length];

        // if the shortest path is found, its length will alway be greater than 0
        for (int i = 0; i < (breadCrumbNodes.length - 1); i++) {
            breadCrumbNodes[i] = new BreadCrumbNode(shortestPath[i].getUrl(), shortestPath[i].getTitle());
        }

        // the last element in the path should be set to currentNode instead.
        if (currentNode.getTitle() != null) {
            breadCrumbNodes[breadCrumbNodes.length - 1] = currentNode;
        } else {
            breadCrumbNodes[breadCrumbNodes.length - 1] = new BreadCrumbNode(currentNode.getUrl(),
                    shortestPath[breadCrumbNodes.length - 1].getTitle());
        }

        return breadCrumbNodes;
    }

    /**
     * <p>
     * Find the shortest path from the site map. The shortest path will be a nodeMatcher array from root nodeMatcher
     * [index 0] to whatever node matcher matches the current node. The totle time complexity is O(n) where n is the
     * number of all the nodes in the site map.
     * </p>
     *
     * @param pageContext the pageContext that can read some session attributes.
     * @param siteMap the site map.
     * @param currentNode the current node.
     *
     * @return the shortest path.
     */
    private NodeMatcher[] getShortestPath(PageContext pageContext, Map siteMap, BreadCrumbNode currentNode) {
        if (siteMap == null) {
            return null;
        }

        // get the reverse mapping of the site map which will help to find all the nodeMatchers' parents
        Map currentReverseSiteMap = null;
        Object object = pageContext.getAttribute(CURRENT_REVERSE_SITEMAP, PageContext.SESSION_SCOPE);

        if (!(object instanceof Map)) {
            currentReverseSiteMap = Helper.getCurrentReverseSiteMap(siteMap);
            pageContext.setAttribute(CURRENT_REVERSE_SITEMAP, currentReverseSiteMap, PageContext.SESSION_SCOPE);
        } else {
            currentReverseSiteMap = (Map) object;
        }

        // try to find the shortest path
        List shortestPath = null;

        for (Iterator iter = siteMap.keySet().iterator(); iter.hasNext();) {
            NodeMatcher nodeMatcher = (NodeMatcher) iter.next();

            if (nodeMatcher.matches(currentNode)) {
                List tempPath = getShortestPath(nodeMatcher, currentReverseSiteMap);

                if ((tempPath != null) && ((shortestPath == null) || (tempPath.size() < shortestPath.size()))) {
                    // update the shortest path
                    shortestPath = tempPath;
                }
            }
        }

        if (shortestPath != null) {
            return (NodeMatcher[]) shortestPath.toArray(new NodeMatcher[shortestPath.size()]);
        }

        // no shortest path can be found
        return null;
    }

    /**
     * <p>
     * Find the shortest path from the site map. The shortest path will be a nodeMatcher array from root nodeMatcher
     * [index 0] to whatever node matcher matches the current node.
     * </p>
     *
     * @param originalMatcher the original matcher.
     * @param currentReverseSiteMap the current reverse site map.
     *
     * @return the shortest path.
     */
    private List getShortestPath(NodeMatcher originalMatcher, Map currentReverseSiteMap) {
        // use bfs to get the shortest path
        Map child = new HashMap();

        // use LinkedList to be more efficient
        List queue = new LinkedList();
        queue.add(originalMatcher);

        // The totle time complexity is O(n) where n is the number of all the nodes in the site map since once a
        // node is reached, it will never be reached again
        child.put(originalMatcher, originalMatcher);

        while (!queue.isEmpty()) {
            NodeMatcher matcher = (NodeMatcher) ((LinkedList) queue).removeFirst();
            List parents = (List) currentReverseSiteMap.get(matcher);

            if (parents == null) {
                // if it is the root, return the shortest path
                List shortestPath = new ArrayList();

                while (!matcher.equals(originalMatcher)) {
                    shortestPath.add(matcher);
                    matcher = (NodeMatcher) child.get(matcher);
                }

                shortestPath.add(originalMatcher);

                return shortestPath;
            }

            for (Iterator iter = parents.iterator(); iter.hasNext();) {
                NodeMatcher nodeMatcher = (NodeMatcher) iter.next();

                if (!child.containsKey(nodeMatcher)) {
                    queue.add(nodeMatcher);
                    child.put(nodeMatcher, matcher);
                }
            }
        }

        return null;
    }

    /**
     * <p>
     * This class handles exceptions produced by the parser during validation of target XML. It extends DefaultHandler,
     * and overrides three methods. Overrided methods will throw SAXParseException.
     * </p>
     *
     * @author pops, TCSDEVELOPER
     * @version 2.0
     */
    private class ErrorChecker extends DefaultHandler {
        /**
         * <p>
         * Creates a new instance of <code>ErrorChecker</code>class.
         * </p>
         */
        public ErrorChecker() {
            // empty constructor, nothing would be done here.
        }

        /**
         * <p>
         * Hander error generated by the parser.
         * </p>
         *
         * @param e SAXParseException generated by the parser.
         *
         * @throws SAXParseException once the method is invoked.
         */
        public void error(SAXParseException e) throws SAXParseException {
            throw e;
        }

        /**
         * <p>
         * Hander warning generated by the parser.
         * </p>
         *
         * @param e SAXParseException generated by the parser.
         *
         * @throws SAXParseException once the method is invoked.
         */
        public void warning(SAXParseException e) throws SAXParseException {
            throw e;
        }

        /**
         * <p>
         * Hander fatal error generated by the parser.
         * </p>
         *
         * @param e SAXParseException generated by the parser.
         *
         * @throws SAXParseException once the method is invoked.
         */
        public void fatalError(SAXParseException e) throws SAXParseException {
            throw e;
        }
    }
}
