/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.nodematchers;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.Helper;
import com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.AbstractNodeMatcher;

import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;


/**
 * <p>
 * This implementation of a NodeMatcher will provide node matching services given a specific regex pattern. This class
 * will return true when the passed node's url matches the specified pattern.
 * </p>
 *
 * <p>
 * <strong>Thread Safety: </strong> This class is thread safe by being immutable.
 * </p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 */
public class RegexNodeMatcher extends AbstractNodeMatcher {
    /**
     * <p>
     * Represents the pattern that will be used to determine if the node matches.  This variable is set  by the
     * constructor, is immutable and can never be null or an empty (trim'd) string,  The pattern will be returned in
     * the getPathPattern getter method
     * </p>
     */
    private final String nodePattern;

    /**
     * <p>
     * Represents the compiled version of the nodePattern.  This variable is set  by the constructor, is immutable and
     * can never be null,  The pattern will be used in the matches method.
     * </p>
     */
    private final Pattern pattern;

    /**
     * <p>
     * Constructs the RegexNodeMatcher from the given title, url and pattern.
     * </p>
     *
     * @param title A non-null, non-empty (trim'd) string describing the default title of the node
     * @param url A non-null, non-empty (trim'd) string representing the url to us
     * @param nodePattern A non-null, non-empty (trim'd) string representing the node pattern to apply
     *
     * @throws IllegalArgumentException if any argument is a null or an empty (trim'd) string
     * @throws BreadCrumbException if the nodePattern is not well formatted.
     */
    public RegexNodeMatcher(String title, String url, String nodePattern) throws BreadCrumbException {
        super(title, url);
        Helper.validateString(nodePattern, "nodePattern");
        this.nodePattern = nodePattern;

        try {
            this.pattern = Pattern.compile(nodePattern);
        } catch (PatternSyntaxException e) {
            throw new BreadCrumbException("The nodePattern is not well formatted.", e);
        }
    }

    /**
     * <p>
     * Returns true if the node's url matches the specified pattern.
     * </p>
     *
     * @param node A non-null node to test
     *
     * @return True if the node matches the regex pattern, false otherwise
     */
    public boolean matches(BreadCrumbNode node) {
        if (node == null) {
            return false;
        }

        return pattern.matcher(node.getUrl()).matches();
    }

    /**
     * <p>
     * Simply the getter method for the node pattern string.
     * </p>
     *
     * @return A non-null, non-empty (trim'd) string representing the regex pattern used to match against nodes
     */
    public String getNodePattern() {
        return nodePattern;
    }

    /**
     * <p>
     * Returns true if the given object matches this node matcher.
     * </p>
     *
     * @param object A possibly null object to test for equality
     *
     * @return True if the object is of type RegexNodeMatcher with a matching nodePattern/title/url
     */
    public boolean equals(Object object) {
        // if o is null or not of RegexNodeMatcher type, return false
        if (!(object instanceof RegexNodeMatcher)) {
            return false;
        }

        // if the nodePattern, title and url all match the other RegexNodeMatcher, return true
        RegexNodeMatcher otherMatcher = (RegexNodeMatcher) object;

        return nodePattern.equals(otherMatcher.nodePattern) && getTitle().equals(otherMatcher.getTitle())
            && getUrl().equals(otherMatcher.getUrl());
    }

    /**
     * <p>
     * Returns the hash code for this object.
     * </p>
     *
     * @return The hashcode of the object
     */
    public int hashCode() {
        return nodePattern.hashCode() ^ getTitle().hashCode() ^ getUrl().hashCode();
    }
}
